#!/bin/bash
##############################################################################
#                                                                            #
#                                                                            #
#              Copyright (c) 2017 IskraTEL                                   #
#                                                                            #
#                                                                            #
#   Name        : VirtCheck.sh                                               #
#                                                                            #
#   Description : Shell script for checking Linux virtualization settings    #
#                                                                            #
#                                                                            #
#   Date        : Dec 2017                                                   #
#                                                                            #
#   Author      : Roman Lotric                                               #
#                                                                            #
##############################################################################

#------------------------------------------------------------------------
#                DEFINES                                                #
#------------------------------------------------------------------------
   RESTORE=$(echo '\033[0m')
   RED=$(echo '\033[00;31m')
   GREEN=$(echo '\033[00;32m')
   YELLOW=$(echo '\033[00;33m')

   cpumaxfreq_default=999999

   sched_min_granularity_ns_MIN=1000000
   sched_min_granularity_ns_MAX=2000000   
   sched_wakeup_granularity_ns_MIN=1500000
   sched_wakeup_granularity_ns_MAX=3000000   
   sched_migration_cost_ns_MIN=400000
   sched_migration_cost_ns_MAX=700000
   swappiness_MIN=10
   swappiness_MAX=30
   dirty_background_ratio_MIN=5
   dirty_background_ratio_MAX=15
   sched_rt_runtime_us_off=-1
   
   KernelVerLimit="3.10"
   CentosVerLimit="7.4"
   WRLVerLimit="7.0.0"
   UbuntuVerLimit="16.4"
   QemuKvmVerLimit="1.9"
   LibvirtVerLimit="3.0"
   GlibVerLimit="2.49"
   GlibcVerLimit="2.25"

#-----------------------------------------------------------------------#
#                FUNCTIONS                                              #
#-----------------------------------------------------------------------#
#-----------------------------------------------------------------------#
#   FUNCTION:     do_version_check()                                    #
#   INPUT:        $1 First version number in dot format (ex. 2.9.2)     #
#                 $2 Second version number                              #
#   OUTPUT:       0 - if equal                                          #
#                 1 - if first version higher                           #
#                 1 - if first version lower                            #
#   DESCRIPTION:  Compare two version strings                           #
#-----------------------------------------------------------------------#
do_version_check() {

   [ "$1" == "$2" ] && return 0

   ver1front=`echo $1 | cut -d "." -f -1`
   ver1back=`echo $1 | cut -d "." -f 2-`

   ver2front=`echo $2 | cut -d "." -f -1`
   ver2back=`echo $2 | cut -d "." -f 2-`

   if [ "$ver1front" != "$1" ] || [ "$ver2front" != "$2" ]; then
       [ "$ver1front" -gt "$ver2front" ] && return 1
       [ "$ver1front" -lt "$ver2front" ] && return 2

       [ "$ver1front" == "$1" ] || [ -z "$ver1back" ] && ver1back=0
       [ "$ver2front" == "$2" ] || [ -z "$ver2back" ] && ver2back=0
       do_version_check "$ver1back" "$ver2back"
       return $?
   else
       [ "$1" -gt "$2" ] && return 1 || return 2
   fi
}

#-----------------------------------------------------------------------#
#   FUNCTION:     testvercomp()                                         #
#   INPUT:        $1 First version number in dot format (ex. 2.9.2)     #
#                 $2 Second version number                              #
#                 $3 criterion =,> or < sign                            #
#   OUTPUT:       1 - if criterion meet                                 #
#                 1 - if criterion not meet                             #
#   DESCRIPTION:  Compare two version strings with criterion            #
#-----------------------------------------------------------------------#
testvercomp () {
    do_version_check $1 $2
    case $? in
	0) op='=';;
	1) op='>';;
	2) op='<';;
    esac
    if [[ $op != $3 ]]
    then
	echo "0"
    else
	echo "1"
    fi
}

#-----------------------------------------------------------------------#
#   FUNCTION:     find_virt_host()                                      #
#   INPUT:        /                                                     #
#   OUTPUT:       Virtual host platform name for guest VM               #
#                 'host' string for host machine                        #
#   DESCRIPTION:  Find virtual platform for guest                       #
#-----------------------------------------------------------------------#
   root=''
   set -u

   have_cpuinfo () {
      test -e "${root}/proc/cpuinfo"
   }

find_virt_host () {   

   echo "#include <stdio.h>                                                             "  > /tmp/cpuid.c
   echo "#include <string.h>                                                            " >> /tmp/cpuid.c
   echo "#if defined(__i386__) || defined(__x86_64__)                                   " >> /tmp/cpuid.c
   echo "static unsigned int                                                            " >> /tmp/cpuid.c
   echo "cpuid (unsigned int eax, char *sig)                                            " >> /tmp/cpuid.c
   echo "{                                                                              " >> /tmp/cpuid.c
   echo "  unsigned int *sig32 = (unsigned int *) sig;                                  " >> /tmp/cpuid.c
   echo "                                                                               " >> /tmp/cpuid.c
   echo "  asm volatile (                                                               " >> /tmp/cpuid.c
   echo "        \"xchgl %%ebx,%1; xor %%ebx,%%ebx; cpuid; xchgl %%ebx,%1\"             " >> /tmp/cpuid.c
   echo "        : \"=a\" (eax), \"+r\" (sig32[0]), \"=c\" (sig32[1]), \"=d\" (sig32[2])" >> /tmp/cpuid.c
   echo "        : \"0\" (eax));                                                        " >> /tmp/cpuid.c
   echo "  sig[12] = 0;                                                                 " >> /tmp/cpuid.c
   echo "                                                                               " >> /tmp/cpuid.c
   echo "  return eax;                                                                  " >> /tmp/cpuid.c
   echo "}                                                                              " >> /tmp/cpuid.c
   echo "static void                                                                    " >> /tmp/cpuid.c
   echo "cpu_sig (void)                                                                 " >> /tmp/cpuid.c
   echo "{                                                                              " >> /tmp/cpuid.c
   echo "  char sig[13];                                                                " >> /tmp/cpuid.c
   echo "  unsigned int base = 0x40000000, leaf = base;                                 " >> /tmp/cpuid.c
   echo "  unsigned int max_entries;                                                    " >> /tmp/cpuid.c
   echo "  memset (sig, 0, sizeof sig);                                                 " >> /tmp/cpuid.c
   echo "  max_entries = cpuid (leaf, sig);                                             " >> /tmp/cpuid.c
   echo "  puts (sig);                                                                  " >> /tmp/cpuid.c
   echo "  if (max_entries > 3 && max_entries < 0x10000) {                              " >> /tmp/cpuid.c
   echo "    for (leaf = base + 0x100; leaf <= base + max_entries; leaf += 0x100) {     " >> /tmp/cpuid.c
   echo "      memset (sig, 0, sizeof sig);                                             " >> /tmp/cpuid.c
   echo "      cpuid (leaf, sig);                                                       " >> /tmp/cpuid.c
   echo "      puts (sig);                                                              " >> /tmp/cpuid.c
   echo "    }                                                                          " >> /tmp/cpuid.c
   echo "  }                                                                            " >> /tmp/cpuid.c
   echo "}                                                                              " >> /tmp/cpuid.c
   echo "#else /* !i386, !x86_64 */                                                     " >> /tmp/cpuid.c
   echo "static void                                                                    " >> /tmp/cpuid.c
   echo "cpu_sig (void)                                                                 " >> /tmp/cpuid.c
   echo "{                                                                              " >> /tmp/cpuid.c
   echo "  /* nothing for other architectures */                                        " >> /tmp/cpuid.c
   echo "}                                                                              " >> /tmp/cpuid.c
   echo "#endif                                                                         " >> /tmp/cpuid.c
   echo "int                                                                            " >> /tmp/cpuid.c
   echo "main()                                                                         " >> /tmp/cpuid.c
   echo "{                                                                              " >> /tmp/cpuid.c
   echo "  cpu_sig ();                                                                  " >> /tmp/cpuid.c
   echo "  return 0;                                                                    " >> /tmp/cpuid.c
   echo "}                                                                              " >> /tmp/cpuid.c

   cc /tmp/cpuid.c -o /tmp/cpuid 2>/dev/null
   if [ $? -eq 0 ]; then
      chmod 755 /tmp/cpuid
      cpuid=$(/tmp/cpuid)                                                  
      rm /tmp/cpuid.c /tmp/cpuid 2>/dev/null
#      if [ -z $cpuid ]; then
      if [[ x$cpuid == x ]]; then
         cpuid="blablabla"
      fi
   else
      cpuid="blablabla"
   fi

   dmi=$(LANG=C dmidecode 2>&1)
   arch=$(uname -p)
   skip_qemu_kvm=false

# Check for VMware.
# cpuid check added by Chetan Loke.
   if [ "$cpuid" = "VMwareVMware" ]; then
      echo vmware
   elif echo "$dmi" | grep -q 'Manufacturer: VMware'; then
      echo vmware
   fi

# Check for Hyper-V.
# http://blogs.msdn.com/b/sqlosteam/archive/2010/10/30/is-this-real-the-metaphysics-of-hardware-virtualization.aspx
   if [ "$cpuid" = "Microsoft Hv" ]; then
      # It is hiperv; now check for QEMU/KVM.
      if echo "$dmi" | grep -q 'Product Name: KVM'; then
         echo hyperv-kvm
      elif echo "$dmi" | grep -q 'Manufacturer: QEMU'; then
         # The test for KVM above failed, so now we know we're
         # not using KVM acceleration.
         echo hyperv-qemu
      elif [ "$arch" = "arm" ] || [ "$arch" = "aarch64" ]; then
         if [ -d "${root}/proc/device-tree" ] &&
            ls "${root}/proc/device-tree" | grep -q "fw-cfg"; then
            # We don't have enough information to determine if we're
            # using KVM acceleration or not.
            echo hyperv-qemu
         fi
      elif [ -d ${root}/proc/device-tree/hypervisor ] &&
         grep -q "linux,kvm" /proc/device-tree/hypervisor/compatible; then
         # We are running as a spapr KVM guest on ppc64
         echo hyperv-kvm
      elif have_cpuinfo && grep -q 'QEMU' "${root}/proc/cpuinfo"; then
         # This is known to fail for qemu with the explicit -cpu
         # option, since /proc/cpuinfo will not contain the QEMU
         # string. QEMU 2.10 added a new CPUID leaf, so this
         # problem only triggered for older QEMU
         echo qemu
      else
         echo hyperv
      fi
   fi

# Check for VirtualPC.
# The negative check for cpuid is to distinguish this from Hyper-V
# which also has the same manufacturer string in the SM-BIOS data.
   if [ "$cpuid" != "Microsoft Hv" ] &&
      echo "$dmi" | grep -q 'Manufacturer: Microsoft Corporation'; then
      echo virtualpc
   fi

# Check for VirtualBox.
# Added by Laurent Léard.
   if echo "$dmi" | grep -q 'Manufacturer: innotek GmbH'; then
      echo virtualbox
   fi

# Check for OpenVZ / Virtuozzo.
# Added by Evgeniy Sokolov.
# /proc/vz - always exists if OpenVZ kernel is running (inside and outside
# container)
# /proc/bc - exists on node, but not inside container.
   if [ -d "${root}/proc/vz" -a ! -d "${root}/proc/bc" ]; then
      echo openvz
   fi

# Check for LXC containers
# http://www.freedesktop.org/wiki/Software/systemd/ContainerInterface
# Added by Marc Fournier
   if [ -e "${root}/proc/1/environ" ] &&
      cat "${root}/proc/1/environ" | tr '\000' '\n' | grep -Eiq '^container='; then
      echo lxc
   fi

# Check for Linux-VServer
#
   if cat "${root}/proc/self/status" | grep -q "VxID: [0-9]*"; then
      echo linux_vserver
      if grep -q "VxID: 0$" "${root}/proc/self/status"; then
         echo linux_vserver-host
      else
         echo linux_vserver-guest
      fi
   fi

# Check for UML.
# Added by Laurent Léard.
   if grep -q 'UML' "${root}/proc/cpuinfo"; then
      echo uml
   fi

# Check for IBM PowerVM Lx86 Linux/x86 emulator.
#
   if grep -q '^vendor_id.*PowerVM Lx86' "${root}/proc/cpuinfo"; then
      echo powervm_lx86
   fi

# Check for Hitachi Virtualization Manager (HVM) Virtage logical partitioning.
   if echo "$dmi" | grep -q 'Manufacturer.*HITACHI' && echo "$dmi" | grep -q 'Product.* LPAR'; then
      echo virtage
   fi

# Check for IBM SystemZ.
#
   if grep -q '^vendor_id.*IBM/S390' "${root}/proc/cpuinfo"; then
      echo ibm_systemz
      if [ -f "${root}/proc/sysinfo" ]; then
         if grep -q 'VM.*Control Program.*z/VM' "${root}/proc/sysinfo"; then
            echo ibm_systemz-zvm
         elif grep -q '^LPAR' "${root}/proc/sysinfo"; then
            echo ibm_systemz-lpar
         else
            # This is unlikely to be correct.
            echo ibm_systemz-direct
         fi
      fi
   fi

# Check for Parallels.
#
   if echo "$dmi" | grep -q 'Vendor: Parallels'; then
      echo parallels
      skip_qemu_kvm=true
   fi

# Check for Xen.
#
   if [ "$cpuid" = "XenVMMXenVMM" ]; then
      echo xen; echo xen-hvm
      skip_qemu_kvm=true
   elif [ -d "${root}/proc/xen" ]; then
      echo xen
      if grep -q "control_d" "${root}/proc/xen/capabilities" 2>/dev/null; then
         echo xen-dom0
      else
         echo xen-domU
      fi
      skip_qemu_kvm=true
   elif [ -f "${root}/sys/hypervisor/type" ] &&
      grep -q "xen" "${root}/sys/hypervisor/type"; then
      # Ordinary kernel with pv_ops.  There does not seem to be
      # enough information at present to tell whether this is dom0
      # or domU.  XXX
      echo xen
   elif [ "$arch" = "ia64" ]; then
      if [ -d "${root}/sys/bus/xen" -a ! -d "${root}/sys/bus/xen-backend" ]; then
         # PV-on-HVM drivers installed in a Xen guest.
         echo xen
         echo xen-hvm
      else
         # There is no virt leaf on IA64 HVM.  This is a last-ditch
         # attempt to detect something is virtualized by using a
         # timing attack.
         virt-what-ia64-xen-rdtsc-test > /dev/null 2>&1
         case "$?" in
            0) ;; # not virtual
            1) # Could be some sort of virt, or could just be a bit slow.
               echo virt
         esac
      fi
   fi

# Check for QEMU/KVM.
#
# Parallels exports KVMKVMKVM leaf, so skip this test if we've already
# seen that it's Parallels.  Xen uses QEMU as the device model, so
# skip this test if we know it is Xen.

   if ! "$skip_qemu_kvm"; then
      if [ "$cpuid" = "KVMKVMKVM" ]; then
   	     echo kvm
      else
         # XXX This is known to fail for qemu with the explicit -cpu
         # option, since /proc/cpuinfo will not contain the QEMU
         # string.  The long term fix for this would be to export
         # another CPUID leaf for non-accelerated qemu.
         if grep -q 'QEMU' "${root}/proc/cpuinfo"; then
   	        echo qemu
         fi
      fi
   fi
}

#------------------------------------------------------------------------
#                KERNEL VERSION                                         #
#------------------------------------------------------------------------
   KernelVersion=$(uname -a | cut -f 3 -d " ")
   KernelVer=$(uname -a | cut -f 3 -d " " | cut -f 1 -d "-")
   KernelVersionMain=$(uname -a | cut -f 3 -d " " | cut -f 1 -d ".")
   runnig_host=0

   my_user=$(whoami)
   echo
   if [[ x$my_user == xroot ]]; then
      p_my_user="${GREEN}$my_user${RESTORE}"
      echo -e "USER=$p_my_user"

      virt_host=$(find_virt_host)
      if [ -z $virt_host ]; then
         echo -e "Running on ${GREEN}HOST${RESTORE} system"
         runnig_host=1
      else
          echo -e "Running  ${GREEN}GUEST${RESTORE} system on ${GREEN}'$virt_host'${RESTORE}"
          runnig_host=0
      fi
   else
      p_my_user="${GREEN}$my_user${YELLOW}"
      echo -e "${RED}"
      echo "###########################################################################"
      echo "#                                                                         #"
      echo "#                   W A R N I N G ! ! !                                   #"
      echo "#                                                                         #"
      echo "###########################################################################"
      echo -e "${YELLOW}"
      echo -e "USER=$p_my_user ....Please start command as root or: sudo $0          "
      echo -e "                                                                      "
      echo -e "Otherwise some results could be incomplete or incorrect               "
      echo -e "${RESTORE}"
   fi



#------------------------------------------------------------------------
#                     GOVERNOR                                          #
#------------------------------------------------------------------------
   nr=$(cat /proc/cpuinfo | grep "processor" | wc -l)
   nr_cores=$(( $nr - 1 ))  

   echo
   echo "GOVERNOR - number of cores: $nr"

   if [ "$KernelVersionMain" -gt 2 ]; then
      for i in $(seq 0 1 $nr_cores); do 
	 if [ -s /sys/devices/system/cpu/cpu$i/cpufreq/scaling_governor ]; then
	    governor=$(cat /sys/devices/system/cpu/cpu$i/cpufreq/scaling_governor)
	    if [ "$governor" = "performance" ]
	    then
	       printf "         checking core%02d:................................${GREEN}PASS${RESTORE}\n" "$i"
	    else
	       printf "         checking core%02d:................................${RED}NOK ($governor)${RESTORE}\n" "$i"
	    fi
	 else 
	       printf "         checking core%02d:................................${YELLOW}SETTING NOT POSSIBLE-No scaling_governor option${RESTORE}\n" "$i"
	 fi
      done;
   else
      printf "         ${YELLOW}CHECKING NOT POSSIBLE: Linux kernel version to old: $KernelVersion${RESTORE}\n"
      printf "         ${YELLOW}No scaling_governor option${RESTORE}\n"
   fi


#------------------------------------------------------------------------
#                     CPU frequency                                     #
#------------------------------------------------------------------------

   if [ "$KernelVersionMain" -gt 2 ]; then
      if hash lscpu 2>/dev/null; then
	 cpumaxfreq=$(lscpu | grep 'CPU max' | awk '{print $4}')
	 if [ x$cpumaxfreq = x ]; then
	    cpumaxfreq=$cpumaxfreq_default
	 fi
         cpumaxfreq=${cpumaxfreq%.*}
         cpumaxfreq=${cpumaxfreq%,*}
      else
	 cpumaxfreq=$cpumaxfreq_default
      fi
      echo
      echo "CPU frequency - $cpumaxfreq MHz"
      CpuList=$(cat /proc/cpuinfo | grep MHz)
      count=0 
      while read -r CpuList; do
	 tmp=$CpuList
	 set -- $tmp
         if [ $# -eq 3 ]; then
            corefreq=$3
         else
	 corefreq=$4
         fi
	 corefreq=${corefreq%.*}
	 if [ "$corefreq" -ge "$cpumaxfreq" ]; then
	    printf "         checking core%02d:................................${GREEN}PASS${RESTORE}\n" "$count"
	 else
	    if [ "$cpumaxfreq" -eq "$cpumaxfreq_default" ]; then
	       printf "         checking core%02d:................................${YELLOW}CHECKING NOT POSSIBLE ($corefreq MHz)${RESTORE}\n" "$count"
	    else
	       printf "         checking core%02d:................................${RED}NOK ($corefreq MHz)${RESTORE}\n" "$count"
	    fi
	 fi
	 count=$(( $count + 1 ))
      done <<< "$CpuList"
   else
      echo "CPU frequency"
      printf "         ${YELLOW}CHECKING NOT POSSIBLE Linux kernel version to old: $KernelVersion${RESTORE}\n"
      corefreq=$(cat /proc/cpuinfo | grep MHz | awk 'NR==1 {print $4}')
      printf "         ${YELLOW}Actual frequency is: %s ${RESTORE}\n" "$corefreq" 
   fi

#------------------------------------------------------------------------
#                  NETWORK INTERFACES                                   #
#------------------------------------------------------------------------
   echo
   echo "NETWORK INTERFACES"

   if hash type netstat 2>/dev/null; then

      NetIntfList=$(netstat -i | cut -f1 -d" " | tail -n+3)

      count=0
      while read -r NetIntfList; do
         tmp=$NetIntfList
         set -- $tmp
         NetIntf=$1
         NetIntf_p="$NetIntf:.............................."

         if hash type ethtool 2>/dev/null; then
            NetIntfSpeed=$(ethtool $NetIntf 2>/dev/null | grep -i speed | cut -f 2 -d " " | cut -f 1 -d "!")
         else
            NetIntfSpeedTmp=$(dmesg | grep eth0 | grep -i "is up")
            if echo "$NetIntfSpeedTmp" | grep -q -e '10000 Mbps' -e '10 Gbps'  -e '10000Mb/s'  -e '10Gb/s'; then
               NetIntfSpeed="10000Mb/s"
            elif echo "$NetIntfSpeedTmp" | grep -q -e '1000 Mbps' -e '1 Gbps'  -e '1000Mb/s'  -e '1Gb/s'; then
               NetIntfSpeed="1000Mb/s"
            elif echo "$NetIntfSpeedTmp" | grep -q -e '100 Mbps' -e '100Mb/s'; then
               NetIntfSpeed="100Mb/s"
            elif echo "$NetIntfSpeedTmp" | grep -q -e '10 Mbps' -e '10Mb/s'; then
               NetIntfSpeed="10Mb/s"
            elif echo "$NetIntfSpeedTmp" | grep -q -e '1 Mbps' -e '1Mb/s'; then
               NetIntfSpeed="1Mb/s"
            else
               NetIntfSpeed="Unknown"
            fi
         fi

         if [ x$NetIntfSpeed == x ] || [ $NetIntfSpeed == "Unknown" ]; then
            NetIntfSpeed="----"
            printf "         ${NetIntf_p:0:25}.......................${YELLOW}      $NetIntfSpeed        ${RESTORE}\n"
         else
            NetIntfSpeedNum=$(echo "$NetIntfSpeed" | sed 's/Mb\/s//g')
            if [ $NetIntfSpeedNum -lt 1000 ]; then
               printf "         ${NetIntf_p:0:25}.......................${YELLOW}LOW SPEED ($NetIntfSpeed)${RESTORE}\n"
            else
               printf "         ${NetIntf_p:0:25}.......................${GREEN}PASS ($NetIntfSpeed)${RESTORE}\n"
            fi
         fi
         count=$(( $count + 1 ))
      done <<< "$NetIntfList"
   else
      printf "         ................................................${YELLOW}CAN NOT FIND ANY INFORMATION{RESTORE}\n"
   fi

#------------------------------------------------------------------------
#                     BIOS virtual. settings                            #
#------------------------------------------------------------------------
   BIOSList=$(egrep -wo 'aes|ept|flexpriority|lm|tpr_shadow|vmx|vnmi|vpid' /proc/cpuinfo | sort | uniq)


   echo
   echo "BIOS virtual settings (aesr,ept,flexpriority,lm,tpr_shadow,vmx,vnmi,vpid)"
   aes=0
   ept=0
   flexpriority=0
   lm=0
   tpr_shadow=0
   vmx=0
   vnmi=0
   vpid=0
   while read -r BIOSList; do
      tmp=$BIOSList
      case "$tmp" in
	 "aes") aes=1
	 ;;
	 "ept") ept=1
	 ;;
	 "flexpriority") flexpriority=1
	 ;;
	 "lm") lm=1
	 ;;
	 "tpr_shadow") tpr_shadow=1
	 ;;
	 "vmx") vmx=1
	 ;;
	 "vnmi") vnmi=1
	 ;;
	 "vpid") vpid=1
	 ;;
      esac
   done <<< "$BIOSList"
   if [ "$aes" -eq 1 ]
      then
	 printf "         AES            :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         AES            :................................${RED}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$ept" -eq 1 ]
      then
	 printf "         EPT            :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         EPT            :................................${RED}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$flexpriority" -eq 1 ]
      then
	 printf "         FLEXPRIORITY   :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         FLEXPRIORITY   :................................${RED}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$lm" -eq 1 ]
      then
	 printf "         LM             :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         LM             :................................${RED}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$tpr_shadow" -eq 1 ]
      then
	 printf "         TPR_SHADOW     :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         TPR_SHADOW     :................................${RED}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$vmx" -eq 1 ]
      then
	 printf "         VMX            :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         VMX            :................................${RED}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$vnmi" -eq 1 ]
      then
	 printf "         VNMI           :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         VNMI           :................................${RED}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$vpid" -eq 1 ]
      then
	 printf "         VPID           :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         VPID           :................................${RED}NOT ENABLED ${RESTORE}\n"
   fi

#------------------------------------------------------------------------
#                     CLOCK SOURCE SETTINGS                             #
#------------------------------------------------------------------------

   echo
   echo "CLOCK SOURCE"

   if [ -r /sys/devices/system/clocksource/clocksource0/current_clocksource ]; then
      ClockSource=$(cat /sys/devices/system/clocksource/clocksource0/current_clocksource)
      if [ x$ClockSource == xtsc ] || [ x$ClockSource == xkvm-clock ]
      then
	 printf "         checking clock source:..........................${GREEN}PASS ($ClockSource)${RESTORE}\n"
      else
	 AvailClockSource=$(cat /sys/devices/system/clocksource/clocksource0/available_clocksource)
	 tsc=0
	 while IFS=' ' read -ra ADDR; do
	    for i in "${ADDR[@]}"; do
	       case "$i" in
		  "tsc=on") tsc=1
		  ;;
	       esac
	    done
	 done <<< "$AvailClockSource"


	 if [ "$tsc" -eq 1 ]
	    then
	       printf "         checking clock source:..........................${RED}NOK ($ClockSource)${RESTORE}\n"
	    else
	       printf "         checking clock source:..........................${YELLOW}No <tsc or kvm-clock> clock source. Available sources are ($AvailClockSource)${RESTORE}\n"
	       printf "         ................................................${YELLOW}Selected is: $ClockSource${RESTORE}\n"
	 fi
      fi
   else 
	 printf "         checking clock source:..........................${YELLOW}CHECKING NOT POSSIBLE-No current_clocksource option${RESTORE}\n"
   fi

#------------------------------------------------------------------------
#                     BOOT PARAMETERS                                   #
#------------------------------------------------------------------------

   echo
   echo "BOOT PARAMETERS"

   BootPar=$(cat /proc/cmdline)
   nohz_off=0
   highres_off=0
   idle_pool=0
   processor_max_cstate=0
   intel_idle_max_cstate=0
   intel_pstate=0
   nosmap=0
   nosoftlockup=0
   mce_ignore_ce=0
   ipv6_disable=0
   intel_iommu=0
   while IFS=' ' read -ra ADDR; do
      for i in "${ADDR[@]}"; do
	 case "$i" in
	    "nohz=off") nohz_off=1
	    ;;
	    "highres=off") highres_off=1
	    ;;
	    "idle=poll") idle_pool=1
	    ;;
	    "processor.max_cstate=0") processor_max_cstate=1
	    ;;
	    "intel_idle.max_cstate=0") intel_idle_max_cstate=1
	    ;;
	    "intel_pstate=disable") intel_pstate=1
	    ;;
	    "nosmap") nosmap=1
	    ;;
	    "nosoftlockup") nosoftlockup=1
	    ;;
	    "mce=ignore_ce") mce_ignore_ce=1
	    ;;
	    "ipv6.disable=1") ipv6_disable=1
	    ;;
	    "intel_iommu=on") intel_iommu=1
	    ;;
	 esac
      done
   done <<< "$BootPar"

   if [ "$nohz_off" -eq 1 ]
      then
	 printf "         nohz=off       :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         nohz=off       :................................${RED}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$highres_off" -eq 1 ]
      then
	 printf "         highres=off    :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         highres=off    :................................${RED}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$idle_pool" -eq 1 ]
      then
	 printf "         idle=poll      :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         idle=poll      :................................${YELLOW}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$processor_max_cstate" -eq 1 ]
      then
	 printf "         processor.max_cstate=0: ........................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         processor.max_cstate=0: ........................${YELLOW}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$intel_idle_max_cstate" -eq 1 ]
      then
	 printf "         intel_idle.max_cstate=0: .......................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         intel_idle.max_cstate=0: .......................${YELLOW}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$intel_pstate" -eq 1 ]
      then
	 printf "         intel_pstate=disable: ..........................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         intel_pstate=disable: ..........................${YELLOW}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$nosmap" -eq 1 ]
      then
	 printf "         nosmap         :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         nosmap         :................................${YELLOW}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$nosoftlockup" -eq 1 ]
      then
	 printf "         nosoftlockup   :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         nosoftlockup   :................................${YELLOW}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$mce_ignore_ce" -eq 1 ]
      then
	 printf "         mce=ignore_ce  :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         mce=ignore_ce  :................................${YELLOW}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$ipv6_disable" -eq 1 ]
      then
	 printf "         ipv6.disable=1 :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         ipv6.disable=1 :................................${YELLOW}NOT ENABLED ${RESTORE}\n"
   fi
   if [ "$intel_iommu" -eq 1 ]
      then
	 printf "         intel_iommu=on :................................${GREEN}ENABLED${RESTORE}\n"
      else
	 printf "         intel_iommu=on :................................${YELLOW}NOT ENABLED ${RESTORE}\n"
   fi

#------------------------------------------------------------------------
#                  KERNEL  PARAMETERS                                   #
#------------------------------------------------------------------------

   echo
   echo "KERNEL  PARAMETERS"
   if [ -r /proc/sys/kernel/sched_min_granularity_ns ]; then
      sched_min_granularity_ns=$(cat /proc/sys/kernel/sched_min_granularity_ns)
      if [ "$sched_min_granularity_ns" -lt "$sched_min_granularity_ns_MIN" ] || [ "$sched_min_granularity_ns" -gt "$sched_min_granularity_ns_MAX" ]; then
	 printf "         sched_min_granularity_ns:.......................${YELLOW}NOT OPTIMAL for performance-latency ($sched_min_granularity_ns)${RESTORE}\n"
      else
	 printf "         sched_min_granularity_ns:.......................${GREEN}PASS ($sched_min_granularity_ns)${RESTORE}\n"
      fi
   else 
      printf "         sched_min_granularity_ns:.......................${YELLOW}NO SUCH SETTING ON YOUR SYSTEM${RESTORE}\n"
   fi

   if [ -r /proc/sys/kernel/sched_wakeup_granularity_ns ]; then
      sched_wakeup_granularity_ns=$(cat /proc/sys/kernel/sched_wakeup_granularity_ns)
      if [ "$sched_wakeup_granularity_ns" -lt "$sched_wakeup_granularity_ns_MIN" ] || [ "$sched_wakeup_granularity_ns" -gt "$sched_wakeup_granularity_ns_MAX" ]; then
	 printf "         sched_wakeup_granularity_ns:....................${YELLOW}NOT OPTIMAL for performance-latency ($sched_wakeup_granularity_ns)${RESTORE}\n"
      else
	 printf "         sched_wakeup_granularity_ns:....................${GREEN}PASS ($sched_wakeup_granularity_ns)${RESTORE}\n"
      fi
   else 
      printf "         sched_wakeup_granularity_ns:....................${YELLOW}NO SUCH SETTING ON YOUR SYSTEM${RESTORE}\n"
   fi

   if [ -r /proc/sys/kernel/sched_migration_cost_ns ]; then
      sched_migration_cost_ns=$(cat /proc/sys/kernel/sched_migration_cost_ns)
      if [ "$sched_migration_cost_ns" -lt "$sched_migration_cost_ns_MIN" ] || [ "$sched_migration_cost_ns" -gt "$sched_migration_cost_ns_MAX" ]; then
	 printf "         sched_migration_cost_ns:........................${YELLOW}NOT OPTIMAL for performance-latency ($sched_migration_cost_ns)${RESTORE}\n"
      else
	 printf "         sched_migration_cost_ns:........................${GREEN}PASS ($sched_migration_cost_ns)${RESTORE}\n"
      fi
   else 
      printf "         sched_migration_cost_ns:........................${YELLOW}NO SUCH SETTING ON YOUR SYSTEM${RESTORE}\n"
   fi

   if [ -r /proc/sys/vm/swappiness ]; then
      swappiness=$(cat /proc/sys/vm/swappiness)
      if [ "$swappiness" -lt "$swappiness_MIN" ] || [ "$swappiness" -gt "$swappiness_MAX" ]; then
	 printf "         swappiness             :........................${YELLOW}NOT OPTIMAL for performance-latency ($swappiness)${RESTORE}\n"
      else
	 printf "         swappiness             :........................${GREEN}PASS ($swappiness)${RESTORE}\n"
      fi
   else 
      printf "         swappiness             :........................${YELLOW}NO SUCH SETTING ON YOUR SYSTEM${RESTORE}\n"
   fi

   if [ -r /proc/sys/vm/dirty_background_ratio ]; then
      dirty_background_ratio=$(cat /proc/sys/vm/dirty_background_ratio)
      if [ "$dirty_background_ratio" -lt "$swappiness_MIN" ] || [ "$dirty_background_ratio" -gt "$swappiness_MAX" ]; then
	 printf "         dirty_background_ratio :........................${YELLOW}NOT OPTIMAL for performance-latency ($dirty_background_ratio)${RESTORE}\n"
      else
	 printf "         dirty_background_ratio :........................${GREEN}PASS ($dirty_background_ratio)${RESTORE}\n"
      fi
   else 
      printf "         dirty_background_ratio :........................${YELLOW}NO SUCH SETTING ON YOUR SYSTEM${RESTORE}\n"
   fi

   if [ -r /sys/kernel/mm/ksm/run ]; then
      ksm=$(cat /sys/kernel/mm/ksm/run)
      if [ "$ksm" -ne 1 ]; then
	 printf "         ksm                    :........................${YELLOW}NOT OPTIMAL for memory consumption ($ksm)${RESTORE}\n"
      else
	 printf "         ksm                    :........................${GREEN}PASS ($ksm)${RESTORE}\n"
      fi
   else 
      printf "         ksm                    :........................${YELLOW}NO SUCH SETTING ON YOUR SYSTEM${RESTORE}\n"
   fi


#------------------------------------------------------------------------
#                  THP FUNCIONALITY                                     #
#------------------------------------------------------------------------

   echo
   echo "THP FUNCIONALITY"

   if [ "$KernelVersionMain" -gt 2 ] &&  [ -r /sys/kernel/mm/transparent_hugepage/enabled ]; then
      thp=$(cat /sys/kernel/mm/transparent_hugepage/enabled)
      always_choice=0
      while IFS=' ' read -ra ADDR; do
	 for i in "${ADDR[@]}"; do
	    case "$i" in
	       "[always]") always_choice=1
	       ;;
	    esac
	 done
      done <<< "$thp"
      if [ "$always_choice" -eq 1 ]
	 then
	    printf "         transparent_hugepage enabled:...................${GREEN}PASS${RESTORE}\n"
	 else
	    printf "         transparent_hugepage enabled:...................${RED}NOK - ($thp)${RESTORE}\n"
      fi

      thp=$(cat /sys/kernel/mm/transparent_hugepage/defrag)
      always_choice=0
      while IFS=' ' read -ra ADDR; do
	 for i in "${ADDR[@]}"; do
	    case "$i" in
	       "[always]") always_choice=1
	       ;;
	    esac
	 done
      done <<< "$thp"
      if [ "$always_choice" -eq 1 ]
	 then
	    printf "         transparent_hugepage defrag:....................${GREEN}PASS${RESTORE}\n"
	 else
	    printf "         transparent_hugepage defrag:....................${RED}NOK - ($thp)${RESTORE}\n"
      fi

      nr_hugepages=$(cat /proc/sys/vm/nr_hugepages)
      if [ "$nr_hugepages" -gt 0 ]
	 then
	    printf "         transparent_hugepage preallocated:..............${GREEN}PASS ($nr_hugepages)${RESTORE}\n"
	 else
	    printf "         transparent_hugepage preallocated:..............${RED}NOK - ($nr_hugepages)${RESTORE}\n"
      fi

      THPFreeList=$(grep HugePages_Free /sys/devices/system/node/node*/meminfo)

      THPFreeNum=0
      while read -r THPFreeList; do
	 tmp=$THPFreeList
	 set -- $tmp
	 tmp1=$4
	 THPFreeNum=$(($THPFreeNum+$tmp1))
      done <<< "$THPFreeList"

      if [ "$THPFreeNum" -gt 0 ]
	 then
	 if [ "$THPFreeNum" -gt 50 ]
	    then
	       printf "         transparent_hugepage Free:......................${GREEN}PASS ($THPFreeNum)${RESTORE}\n"
	    else
	       printf "         transparent_hugepage Free:......................${YELLOW}LOW NUMBER OF FREE PAGES($THPFreeNum)${RESTORE}\n"
	 fi
	 else
	    printf "         transparent_hugepage Free:......................${RED}NOK - ($THPFreeNum)${RESTORE}\n"
      fi
   else
      printf "         ${YELLOW}CHECKING NOT POSSIBLE: Linux kernel version to old: $KernelVersion${RESTORE}\n"
      if ! [ -r /sys/kernel/mm/transparent_hugepage/enabled ]; then
	 printf "         ${YELLOW}No THP functionality${RESTORE}\n"
      fi
   fi

#------------------------------------------------------------------------
#                  SW  VERSIONS                                         #
#------------------------------------------------------------------------

   echo
   echo "SW VERSIONS"

#kernel 
   ret_val=$(testvercomp $KernelVer $KernelVerLimit '>')
   if [ $ret_val -eq 1 ]; then
      printf "         Linux Kernel:...................................${GREEN}PASS ($KernelVersion)${RESTORE}\n"
   else
      printf "         Linux Kernel:...................................${YELLOW}LINUX KERNEL VERSION LOW ($KernelVersion)${RESTORE}\n"
   fi

#distro
#CentOS
   OS_TYPE="UNSUPPORTED DISTRIBUTION"
   if [ `cat /etc/*-release | grep "^NAME" | grep "CentOS Linux" | uniq | wc -l` -eq 1 ] || [ `cat /etc/*-release |  grep "CentOS release" | uniq | wc -l` -eq 1 ]; then
      if [ `cat /etc/*-release | grep "^NAME" | grep "CentOS Linux" | uniq | wc -l` -eq 1 ]; then
	 OS_TYPE=$(cat /etc/*-release| grep "CentOS Linux" |  grep "release " | uniq)
	 CentosVer=$(cat /etc/*-release | grep "CentOS Linux" |  grep "release " | uniq | cut -f 4 -d " ")
	 ret_val=$(testvercomp $CentosVer $CentosVerLimit '>')
	 if [ $ret_val -eq 1 ]; then
	    printf "         OS type:........................................${GREEN}PASS ($OS_TYPE)${RESTORE}\n"
	 else
	    printf "         OS type:........................................${YELLOW}VERSION IS LOW ($OS_TYPE)${RESTORE}\n"
	 fi
      else
	 OS_TYPE=$(cat /etc/*-release |  grep "CentOS release" | uniq)
	 printf "         OS type:........................................${RED}OLD VERSION ($OS_TYPE)${RESTORE}\n"
      fi
   else
#WRL
      if [ `cat /etc/*-release | grep "Wind River Linux" | uniq | wc -l` -gt 0 ]; then
         if [ `cat /etc/*-release | grep "^NAME" | grep "Wind River Linux" | uniq | wc -l` -eq 1 ]; then
            OS_TYPE=$(cat /etc/*-release | grep "PRETTY_NAME" | cut -f 2 -d "\"")
            WRLVer=$(cat /etc/*-release | grep "PRETTY_NAME" | cut -f 2 -d "\"" | cut -f 5 -d " ")
	    if [ x$WRLVer = x ]; then 
               WRLVer=$(cat /etc/*-release | grep "PRETTY_NAME" | cut -f 2 -d "\"" | cut -f 4 -d " ")
	    fi
            ret_val=$(testvercomp $WRLVer $WRLVerLimit '>')
            if [ $ret_val -eq 1 ]; then
               printf "         OS type:........................................${GREEN}PASS ($OS_TYPE)${RESTORE}\n"
            else
               printf "         OS type:........................................${YELLOW}VERSION IS LOW ($OS_TYPE)${RESTORE}\n"
            fi
         else
            if [ `cat /etc/*-release | grep "Wind River Linux" | uniq | wc -l` -eq 1 ]; then
               OS_TYPE=$(cat /etc/*-release | grep "Wind River Linux" | cut -f 2 -d "\"")
               WRLVer=$(cat /etc/*-release | grep "Wind River Linux" | cut -f 2 -d "\"" | cut -f 4 -d " ")
               ret_val=$(testvercomp $WRLVer $WRLVerLimit '>')
               if [ $ret_val -eq 1 ]; then
                  printf "         OS type:........................................${GREEN}PASS ($OS_TYPE)${RESTORE}\n"
               else
                  printf "         OS type:........................................${YELLOW}VERSION IS LOW ($OS_TYPE)${RESTORE}\n"
               fi
            else
	       printf "         OS type:........................................${RED}OS is  ($OS_TYPE)${RESTORE}\n"
            fi
         fi
      else
#Ubuntu
	 if [ `cat /etc/*-release | grep "^NAME" | grep "Ubuntu" | uniq | wc -l` -eq 1 ]; then
	    OS_TYPE=$(cat /etc/*-release | grep "PRETTY_NAME" | cut -f 2 -d "\"")
	    UbuntuVer=$(cat /etc/*-release | grep "PRETTY_NAME" | cut -f 2 -d "\"" | cut -f 2 -d " ")
	    ret_val=$(testvercomp $UbuntuVer $UbuntuVerLimit '>')
	    if [ $ret_val -eq 1 ]; then
	       printf "         OS type:........................................${GREEN}PASS ($OS_TYPE)${RESTORE}\n"
	    else
	       printf "         OS type:........................................${YELLOW}VERSION IS LOW ($OS_TYPE)${RESTORE}\n"
	    fi
	 else
#MontaVista
	    if [ `cat /etc/*-release | grep "MontaVista Linux" | uniq | wc -l` -eq 1 ]; then
	       OS_TYPE=$(cat /etc/*-release)
	       printf "         OS type:........................................${YELLOW}OLD DISTRIBUTION ($OS_TYPE)${RESTORE}\n"
	    else
	       printf "         OS type:........................................${RED}OS is  ($OS_TYPE)${RESTORE}\n"
	    fi
	 fi
      fi
   fi
   
#qemu-kvm
    if [ -r /usr/libexec/qemu-kvm ]; then
       QemuKvm=$(/usr/libexec/qemu-kvm -version | awk -F '[()]' '{print $2}')
       QemuKvmVer=$(/usr/libexec/qemu-kvm -version | awk -F '[()]' '{print $2}' | cut -f 3 -d "-")
       ret_val=$(testvercomp $QemuKvmVer $QemuKvmVerLimit '>')
       if [ $ret_val -eq 1 ]; then
         printf "         QEMU-KVM:.......................................${GREEN}PASS ($QemuKvm)${RESTORE}\n"
      else
         printf "         QEMU-KVM:.......................................${YELLOW}VERSION IS LOW ($QemuKvm)${RESTORE}\n"
      fi
   else
      if [ -e /usr/bin/qemu-system-x86_64 ]; then
         QemuKvm=$(/usr/bin/qemu-system-x86_64 -version 2>/dev/null )
         QemuKvm=$(echo $QemuKvm | cut -f 4 -d " " | cut -f 1 -d ",")
         ret_val=$(testvercomp $QemuKvm $QemuKvmVerLimit '>')
         if [ $ret_val -eq 1 ]; then
            printf "         QEMU-KVM:.......................................${GREEN}PASS ($QemuKvm)${RESTORE}\n"
         else
            printf "         QEMU-KVM:.......................................${YELLOW}VERSION IS LOW ($QemuKvm)${RESTORE}\n"
         fi
      else
         printf "         QEMU-KVM:.......................................${YELLOW}QEMU-KVM NOT INSTALLED (on standard PATH)${RESTORE}\n"
      fi
   fi

#libvirt
   if hash type libvirtd 2>/dev/null; then
      LibvirtVer=$(libvirtd --version | cut -f 3 -d " ")
      ret_val=$(testvercomp $LibvirtVer $LibvirtVerLimit '>')
      if [ $ret_val -eq 1 ]; then
         printf "         LIBVIRT:........................................${GREEN}PASS ($LibvirtVer)${RESTORE}\n"
      else
         printf "         LIBVIRT:........................................${YELLOW}VERSION IS LOW ($LibvirtVer)${RESTORE}\n"
      fi
   else
      printf "         LIBVIRT:........................................${YELLOW}LIBVIRT NOT INSTALLED (on standard PATH)${RESTORE}\n"
   fi

#glib
   GlibVer=$(find /usr/lib64 -type f | grep libglib- 2>/dev/null | awk 'NR==1' | tr ' ' '\n' | tail -n1 | cut -d'-' -f 2 | awk -F '.' '{print $1"."$5/100"."$6}')
   if ! [ x$GlibVer = x ]; then
      if hash type glib-compile-resources 2>/dev/null; then
         command_path=$(type glib-compile-resources)
         if [ -n command_path ]; then
            glib-compile-resources --version &>/dev/null
            if [ $? -eq 0 ]; then
               GlibVer_gcr=$(glib-compile-resources --version)
               ret_val=$(testvercomp $GlibVer  $GlibVer_gcr '=')
               if [ $ret_val -eq 1 ]; then
                  Glib_minor=$(echo ${GlibVer} | cut -d "." -f 3)
                  if ! [ x$Glib_minor = x ]; then
                     ret_val=$(testvercomp $GlibVer $GlibVerLimit '>')
                     if [ $ret_val -eq 1 ]; then
                        printf "         GLIB:...........................................${GREEN}PASS ($GlibVer)${RESTORE}\n"
                     else
                        printf "         GLIB:...........................................${YELLOW}VERSION IS LOW ($GlibVer)${RESTORE}\n"
                     fi
                  else
                     printf "         GLIB:...........................................${RED}CAN NOT FIND GLIB VERSION${RESTORE}\n"
                  fi
               else #different versions
                  Glib_minor=$(echo ${GlibVer} | cut -d "." -f 3)
                  if ! [ x$Glib_minor = x ]; then
                     ret_val=$(testvercomp $GlibVer $GlibVerLimit '>')
                     if [ $ret_val -eq 1 ]; then
                        printf "         GLIB:...........................................${GREEN}PASS ($GlibVer)${RESTORE}\n"
                     else
                        printf "         GLIB:...........................................${YELLOW}VERSION IS LOW ($GlibVer)${RESTORE}\n"
                     fi
                  else
                     Glib_gcr_minor=$(echo ${GlibVer_gcr} | cut -d "." -f 3)
                     if ! [ x$Glib_gcr_minor = x ]; then
                        ret_val=$(testvercomp $GlibVer_gcr $GlibVerLimit '>')
                        if [ $ret_val -eq 1 ]; then
                           printf "         GLIB:...........................................${GREEN}PASS ($GlibVer_gcr)${RESTORE}\n"
                        else
                           printf "         GLIB:...........................................${YELLOW}VERSION IS LOW ($GlibVer_gcr)${RESTORE}\n"
                        fi
                     else
                        printf "         GLIB:...........................................${RED}CAN NOT FIND GLIB VERSION${RESTORE}\n"
                     fi
                  fi 
               fi
            else #glib-compile-resources command  version problem
               Glib_minor=$(echo ${GlibVer} | cut -d "." -f 3)
               if ! [ x$Glib_minor = x ]; then
                  ret_val=$(testvercomp $GlibVer $GlibVerLimit '>')
                  if [ $ret_val -eq 1 ]; then
                     printf "         GLIB:...........................................${GREEN}PASS ($GlibVer)${RESTORE}\n"
                  else
                     printf "         GLIB:...........................................${YELLOW}VERSION IS LOW ($GlibVer)${RESTORE}\n"
                  fi
               else
                  printf "         GLIB:...........................................${RED}CAN NOT FIND GLIB VERSION${RESTORE}\n"
               fi
            fi
         else #no glib-compile-resources command_path
            Glib_minor=$(echo ${GlibVer} | cut -d "." -f 3)
            if ! [ x$Glib_minor = x ]; then
               ret_val=$(testvercomp $GlibVer $GlibVerLimit '>')
               if [ $ret_val -eq 1 ]; then
                  printf "         GLIB:...........................................${GREEN}PASS ($GlibVer)${RESTORE}\n"
               else
                  printf "         GLIB:...........................................${YELLOW}VERSION IS LOW ($GlibVer)${RESTORE}\n"
               fi
            else
               printf "         GLIB:...........................................${RED}CAN NOT FIND GLIB VERSION${RESTORE}\n"
            fi
         fi
      else #no glib-compile-resources command
         Glib_minor=$(echo ${GlibVer} | cut -d "." -f 3)
         if ! [ x$Glib_minor = x ]; then
            ret_val=$(testvercomp $GlibVer $GlibVerLimit '>')
            if [ $ret_val -eq 1 ]; then
               printf "         GLIB:...........................................${GREEN}PASS ($GlibVer)${RESTORE}\n"
            else
               printf "         GLIB:...........................................${YELLOW}VERSION IS LOW ($GlibVer)${RESTORE}\n"
            fi
         else
            printf "         GLIB:...........................................${RED}CAN NOT FIND GLIB VERSION${RESTORE}\n"
         fi
      fi
   else
      if hash type glib-compile-resources 2>/dev/null; then
         command_path=$(type glib-compile-resources)
         if [ -n command_path ]; then
            glib-compile-resources --version &>/dev/null
            if [ $? -eq 0 ]; then
               GlibVer=$(glib-compile-resources --version)
               ret_val=$(testvercomp $GlibVer $GlibVerLimit '>')
               if [ $ret_val -eq 1 ]; then
                  printf "         GLIB:...........................................${GREEN}PASS ($GlibVer)${RESTORE}\n"
               else
                  printf "         GLIB:...........................................${YELLOW}VERSION IS LOW ($GlibVer)${RESTORE}\n"
               fi
            else
               glib-mkenums --version &>/dev/null
               if [ $? -eq 0 ]; then
                  GlibVer=$(glib-mkenums --version 2>/dev/null | grep version | cut -d "-" -f 3)
                  ret_val=$(testvercomp $GlibVer $GlibVerLimit '>')
                  if [ $ret_val -eq 1 ]; then
                     printf "         GLIB:...........................................${GREEN}PASS ($GlibVer)${RESTORE}\n"
                  else
                     printf "         GLIB:...........................................${YELLOW}VERSION IS LOW ($GlibVer)${RESTORE}\n"
                  fi
               else
                  printf "         GLIB:...........................................${YELLOW}GLIB NOT INSTALLED (on standard PATH)${RESTORE}\n"
               fi
            fi
         else
            printf "         GLIB:...........................................${YELLOW}GLIB NOT INSTALLED (on standard PATH)${RESTORE}\n"
         fi
      else
         printf "         GLIB:...........................................${YELLOW}GLIB NOT INSTALLED (on standard PATH)${RESTORE}\n"
      fi
   fi
#glibc
   GlibcVer=$(ldd --version | awk '/ldd/{print $NF}')
   ret_val=$(testvercomp $GlibcVer $GlibcVerLimit '>')
   if [ $ret_val -eq 1 ]; then
      printf "         GLIBC:..........................................${GREEN}PASS ($GlibcVer)${RESTORE}\n"
   else
      printf "         GLIBC:..........................................${YELLOW}VERSION IS LOW ($GlibcVer)${RESTORE}\n"
   fi

#------------------------------------------------------------------------
#                  QEMU-KVM                                             #
#------------------------------------------------------------------------

   echo
   echo "QEMU-KVM"
   QEMUKVM=$(ps axf | grep qemu | grep format )
   if [ ${#QEMUKVM} -gt 0 ]; then


      echo -e "         VMName\t\t                            \t VMformat\tvirtio\tHostCpu\t"
      echo -e "         -------------------------------------------------------------------------------"
      count=0
      while read -r QEMUKVM; do
         tmp=$QEMUKVM
         count=$(( $count + 1 ))
#VM name
         searchstring="-name"
         rest=${tmp#*$searchstring}
         posStart=$(( ${#tmp} - ${#rest} - ${#searchstring} ))
         VMName=$(echo ${tmp:posStart} | cut -d " " -f 2)
         VMName="$VMName:..............................."

#VM format
         searchstring="format"
         rest=${tmp#*$searchstring}
         posStart=$(( ${#tmp} - ${#rest} - ${#searchstring} ))
         VMformat=$(echo ${tmp:posStart} | cut -d "=" -f 2 |cut -d " " -f 1 |cut -d "," -f 1 )
         if [[ x$VMformat == xraw ]]; then
            VMformat="${GREEN}$VMformat${RESTORE}"
         else
            VMformat="${YELLOW}$VMformat${RESTORE}"
         fi
#virtio
         searchstring="virtio-net-pci"
         rest=${tmp#*$searchstring}
         find_flag=$((${#tmp} - ${#rest} ))
         if [ $find_flag -eq 0 ]; then
            VMvirtio="${RED}0${RESTORE}"
         else
            VMvirtio="${GREEN}1${RESTORE}"
         fi
#cpu mode
         searchstring="-cpu"
         rest=${tmp#*$searchstring}
         find_flag=$((${#tmp} - ${#rest} ))
         if [ $find_flag -eq 0 ]; then
            VMHostCpu="${RED}0${RESTORE}"
         else
            VMHostCpu="${GREEN}1${RESTORE}"
         fi
         echo -e "         ${VMName:0:25}...................\t $VMformat\t\t$VMvirtio\t$VMHostCpu"
      done <<< "$QEMUKVM"
   else
      printf "         QEMU-KVM:.......................................${YELLOW}NO VMs RUNNING${RESTORE}\n"
   fi 

#------------------------------------------------------------------------
#                  OTHERS                                               #
#------------------------------------------------------------------------

   echo
   echo "OTHER SETTINGS"

#CPU throttling
   if [ -r /proc/sys/kernel/sched_rt_runtime_us ]; then
      sched_rt_runtime_us=$(cat /proc/sys/kernel/sched_rt_runtime_us)
      if [ "$sched_rt_runtime_us" -eq "$sched_rt_runtime_us_off" ]; then
	 printf "         CPU throttling:.................................${GREEN}PASS ($sched_rt_runtime_us)${RESTORE}\n"
      else
	 printf "         CPU throttling:.................................${YELLOW}NOT OPTIMAL for performance ($sched_rt_runtime_us)${RESTORE}\n"
	 printf "                                                         ${YELLOW}NOT possible to set real-time priority!!!${RESTORE}\n"
      fi
   else 
      printf "         CPU throttling:.................................${YELLOW}NO SUCH SETTING ON YOUR SYSTEM${RESTORE}\n"
   fi
#SRIOV
   if hash type lspci 2>/dev/null; then
      SRIOVList=$(lspci | grep "Ethernet controller:")
      SRIOVPossible=0
      NumVFSAll=0
      if ! [ -z "$SRIOVList" ]; then
      SRIOVPorts="PCI ports:"
      while read -r SRIOVList; do
	 tmp=$SRIOVList
	 set -- $tmp
	 PCIAddress=$1
	 SRIOVText=$(lspci -s $PCIAddress -vvv 2>/dev/null  | grep -i "Single Root I/O Virtualization" | awk 'NR==1' | tr ' ' '\n' | tail -n1)
	 if ! [ x$SRIOVText = x ]; then
	    SRIOVPossible=1
                NumVFS=$(cat /sys/bus/pci/devices/0000:$PCIAddress/sriov_numvfs 2>/dev/null)
                NumVFSAll=$(( $NumVFSAll + $NumVFS ))
                
	    SRIOVPorts="$SRIOVPorts $PCIAddress(NumVFS:$NumVFS)"
	 fi
      done <<< "$SRIOVList"
      fi
      if [ "$SRIOVPossible" -eq 1 ]
	 then
                if [ "$NumVFSAll" -gt 0 ]; then
	       printf "         SRIOV:..........................................${GREEN}PASS ($SRIOVPorts)${RESTORE}\n"
                else
	       printf "         SRIOV:..........................................${YELLOW}POSSIBLE, BUT NO VIRTUAL PORTS ENABLED ($SRIOVPorts)${RESTORE}\n"
                fi
	 else
	    printf "         SRIOV:..........................................${YELLOW}FUNCTIONALITY NOT POSSIBLE${RESTORE}\n"
      fi
       else
          printf "         SRIOV:..........................................${YELLOW}FUNCTIONALITY NOT POSSIBLE${RESTORE}\n"
   fi

